#ifndef GPU_H
#define GPU_H
//-------------------------------------------------------
#include <types.h>
//-------------------------------------------------------
//---- PAD definitions ----------------------------------
//-------------------------------------------------------
#define PAD_L2	     (1 << 0)
#define PAD_R2	     (1 << 1)
#define PAD_L1	     (1 << 2)
#define PAD_R1	     (1 << 3)
#define PAD_TRIANGLE (1 << 4)
#define PAD_CIRCLE   (1 << 5)
#define PAD_CROSS    (1 << 6)
#define PAD_SQUARE   (1 << 7)
#define PAD_SELECT   (1 << 8)
#define PAD_START    (1 << 11)
#define PAD_UP       (1 << 12)
#define PAD_RIGHT    (1 << 13)
#define PAD_DOWN     (1 << 14)
#define PAD_LEFT     (1 << 15)
//-------------------------------------------------------
#define MODE_NTSC (0)
#define MODE_PAL  (1)
#define MODE_AUTO (2)
#define NULL 	  (0x00000000)
#define TRUE 	  (1)
#define FALSE 	  (0)
#define ONE 	  (4096)
//-------------------------------------------------------
#define ENV_W256 (0 << 6 | 0)
#define ENV_W320 (0 << 6 | 1)
#define ENV_W512 (0 << 6 | 2)
#define ENV_W640 (0 << 6 | 3)
#define ENV_W384 (1 << 6 | 0)
#define ENV_H240 (0 << 2)
#define ENV_H480 (1 << 2)
//-------------------------------------------------------
#define GetClut(x, y) ( (y << 6) | ((x >> 4) & 0x3f) )
#define GetTPage(tp, abr, x, y) \
 ( (tp & 0x3) << 7 | (abr & 0x3) << 5 | (y & 0x100) >> 4 | (x & 0x3ff) >> 6 )
//-------------------------------------------------------
#define	GPU_COM(com, r, g, b)  ( (b << 16) | (g << 8) | r | (com << 24) )
#define	H2L(x, y)              ( (x & 0xffff) | (y << 16) )
#define	HB2L(id, u, v)         ( (id << 16) | (v << 8) | u )
//-------------------------------------------------------
typedef struct {
  short x, y;
  short w, h;
} RECT;

typedef struct {
  u_long id;
  u_long flag;
} TIMHEADER;

typedef struct {
  u_long datalen;
  u_long vrampos;
  u_long size;
  u_long data;
} TIMDATA;

typedef struct {
  RECT   disp;
  RECT   screen;
  u_char isinter;
  u_char isrgb24;
} DISPENV;

typedef struct {
  RECT 	  clip;
  short   ofs[2];
  RECT    tw;
  u_char  tpage;
  u_short dtd;
  u_short dfe;
  u_char  isbg;
  u_char  r0, g0, b0;
} DRAWENV;

typedef struct {
  u_long  pmode;
  short   px, py;
  u_short pw, ph;
  u_short clutid;
  u_short tpage;
} GsIMAGE;

typedef struct {
  u_char  r, g, b, p;
  short   x, y;
  u_short w, h;
  u_char  u, v;
  u_short clutid;
  u_short tpage;
  short   mx, my;
  short   scalex, scaley;
  long    rotate;
  long    trans;
} GsSPRITE;

typedef struct {
  u_char  r, g, b, p;
  short   x, y;
  long    trans;
} GsDOT;

typedef struct {
  u_char  r, g, b, p;
  short   x, y;
  u_short w, h;
  long    trans;
} GsTILE;

typedef struct {
  u_char r, g, b, p;
  short  x0, y0;
  short  x1, y1;
  long   trans;
} GsLINE;

typedef struct {
  u_char r0, g0, b0, p0;
  short  x0, y0;
  u_char r1, g1, b1, p1;
  short  x1, y1;
  long   trans;
} GsGLINE;

typedef struct {
  u_char r, g, b, p;
  short  x0, y0;
  short  x1, y1;
  short  x2, y2;
  long   trans;
} GsPOLY3;

typedef struct {
  u_char r0, g0, b0, p0;
  short  x0, y0;
  u_char r1, g1, b1, p1;
  short  x1, y1;
  u_char r2, g2, b2, p2;
  short  x2, y2;
  long   trans;
} GsGPOLY3;

typedef struct {
  u_char  r, g, b, p0;
  short   x0, y0;
  u_char  u0, v0;
  u_short clutid;
  short   x1, y1;
  u_char  u1, v1;
  u_short tpage;
  short   x2, y2;
  u_char  u2, v2;
  u_short pad2;
  long    trans;
} GsTPOLY3;

typedef struct {
  u_char  r0, g0, b0, p0;
  short   x0, y0;
  u_char  u0, v0;
  u_short clutid;
  u_char  r1, g1, b1, p1;
  short   x1, y1;
  u_char  u1, v1;
  u_short tpage;
  u_char  r2, g2, b2, p2;
  short   x2, y2;
  u_char  u2, v2;
  u_short pad2;
  long    trans;
} GsGTPOLY3;

typedef struct {
  u_char r, g, b, p;
  short  x0, y0;
  short  x1, y1;
  short  x2, y2;
  short  x3, y3;
  long   trans;
} GsPOLY4;

typedef struct {
  u_char r0, g0, b0, p0;
  short  x0, y0;
  u_char r1, g1, b1, p1;
  short  x1, y1;
  u_char r2, g2, b2, p2;
  short  x2, y2;
  u_char r3, g3, b3, p3;
  short  x3, y3;
  long   trans;
} GsGPOLY4;

typedef struct {
  u_char  r, g, b, p;
  short   x0, y0;
  u_char  u0, v0;
  u_short clutid;
  short   x1, y1;
  u_char  u1, v1;
  u_short tpage;
  short   x2, y2;
  u_char  u2, v2;
  u_short pad2;
  short   x3, y3;
  u_char  u3, v3;
  u_short pad3;
  long    trans;
} GsTPOLY4;

typedef struct {
  u_char  r0, g0, b0, p0;
  short   x0, y0;
  u_char  u0, v0;
  u_short clutid;
  u_char  r1, g1, b1, p1;
  short   x1, y1;
  u_char  u1, v1;
  u_short tpage;
  u_char  r2, g2, b2, p2;
  short   x2, y2;
  u_char  u2, v2;
  u_short pad2;
  u_char  r3, g3, b3, p3;
  short   x3, y3;
  u_char  u3, v3;
  u_short pad3;
  long    trans;
} GsGTPOLY4;

DISPENV GsDISPENV;
DRAWENV GsDRAWENV;
//-------------------------------------------------------
typedef struct {
  short m[3][3];
  long t[3];
} MATRIX;

typedef struct {
  short x, y;
} SVECTOR2D;

typedef struct {
  short x, y, z, pad;
} SVECTOR3D;

typedef struct {
  long x, y, z;
} VECTOR;

typedef struct {
  u_long id;
  u_long flags;
  u_long nobj;
} TMDHEADER;

typedef struct {
  u_int x, y, z, pad;       /* Vertices of the objects */
} TMDVERTEX;

typedef struct {
  u_int x, y, z, pad;       /* Normals of the object */
} TMDNORMAL;

typedef struct {
  u_char oLen;     /* Word length of output buffer after processing */
  u_char iLen;     /* Word length of input parameter data (after header) */
  u_char flag;     /* Flags */
  u_char mode;     /* Primitive type and options */
} TMDPRIMHDR;

typedef struct {
  u_long * vert_top;		// start address of vertices
  u_long n_vert;		// number of vertices
  u_long * normal_top;     	// start address of normals
  u_long n_normal;		// number of normals
  u_long * primitive_top; 	// start address of primitives
  u_long n_primitive;		// number of primitives
  long   scale;			// scale value
} TMDOBJECT;

typedef struct {
  int    vx, vy, vz;
  u_char r, g, b, pad;
} GsF_LIGHT;

typedef struct {
  long        x, y, z;
  long        rx, ry, rz;
  TMDHEADER * model;
} GsOBJECT;

typedef struct {
  long x, y, z;
  long ax, ay, az;
} GsCAMERA;
//-------------------------------------------------------
void  LoadImage(RECT * rect, u_long * p);
long  LoadTim(TIMHEADER * tim);
void  GsGetTimInfo(TIMHEADER * tim, GsIMAGE * img);
//-------------------------------------------------------
long  SetVideoMode(long mode);
long  GetVideoMode(void);
void  ResetGraph(int mode);
void  GsInitGraph(int x_res, int y_res, int intl, int dither, int vram);
void  GsInitGraphQuick(int x_res, int y_res);
DRAWENV * PutDrawEnv(DRAWENV * env);
DISPENV * PutDispEnv(DISPENV * env);
//-------------------------------------------------------
void  GsDefDispBuff(int x0, int y0, int x1, int y1);
void  GsSwapDispBuff(void);
void  GsDrawOT(u_long * pGP);
void  GsSortClear(int r, int g, int b);
int   GsGetActiveBuff(void);
void  GsSetWorkBase(u_char * gp_addr);
//-------------------------------------------------------
void  GsSortDot(GsDOT * dot);
void  GsSortGLine(GsGLINE * line);
void  GsSortLine(GsLINE * line);
void  GsSortSprite(GsSPRITE * sprite);
void  GsSortTile(GsTILE * tile);
//-------------------------------------------------------
void  GsSortPoly3(GsPOLY3 * poly);
void  GsSortGPoly3(GsGPOLY3 * poly);
void  GsSortTPoly3(GsTPOLY3 * poly);
void  GsSortGTPoly3(GsGTPOLY3 * poly);
void  GsSortPoly4(GsPOLY4 * poly);
void  GsSortGPoly4(GsGPOLY4 * poly);
void  GsSortTPoly4(GsTPOLY4 * poly);
void  GsSortGTPoly4(GsGTPOLY4 * poly);
//-------------------------------------------------------
void FntLoad(u_char num, TIMHEADER * new_font);
void FntColor(u_char num, u_char r, u_char g, u_char b);
void FntPrint(u_char num, int x, int y, char * fmt, ...);
//-------------------------------------------------------
void GsSortObject(GsOBJECT * object);
long GsSortObject2(GsOBJECT * object);
void GsInit3d (void);
void GsSet3dOffset(int x_res, int y_res);
void GsSetFogColor(u_char r, u_char g, u_char b);
void LoadTmd(TMDHEADER * tmd);
void GsResetMatrix(MATRIX * matrix);
void MulMatrix0(MATRIX * m0, MATRIX * m1, MATRIX * m2);
void GsSetFlatLight(int id, GsF_LIGHT * light);
void GsSetH(int h);
void GsSetXYZ(int x, int y, int z);
void GsSetView (GsCAMERA * camera);
//-------------------------------------------------------

#endif // GPU_H


